package missing;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Arrays;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class InsertionDialog extends JDialog{
    private MainFrame frame;
    
    private JTextField name;
    private JList knownNames;
    
    private JButton okButton;
    private JButton cancelButton;
    
    public InsertionDialog( MainFrame frame ){
        super( frame );
        this.frame = frame;
        setModal( true );
        setTitle( "Add Dockable" );
        
        name = new JTextField();
        
        knownNames = new JList();
        knownNames.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        knownNames.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( ListSelectionEvent e ) {
                String text = (String)knownNames.getSelectedValue();
                if( text != null ){
                    name.setText( text );
                }
            }
        });
        name.getDocument().addDocumentListener( new DocumentListener(){
            public void changedUpdate( DocumentEvent e ) {
                updateEnabledState();
            }
            public void insertUpdate( DocumentEvent e ) {
                updateEnabledState();
            }
            public void removeUpdate( DocumentEvent e ) {
                updateEnabledState();
            }            
        });
        
        setDefaultCloseOperation( DO_NOTHING_ON_CLOSE );
        addWindowListener( new WindowAdapter(){
            @Override
            public void windowClosing( WindowEvent e ) {
                cancelPressed();
            }
        });
        
        okButton = new JButton( "Add" );
        okButton.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent e ) {
                okPressed();
            }
        });
        
        cancelButton = new JButton( "Cancel" );
        cancelButton.addActionListener( new ActionListener(){
            public void actionPerformed( ActionEvent e ) {
                cancelPressed();
            }
        });
        
        JPanel buttons = new JPanel( new GridLayout( 1, 2 ));
        buttons.add( okButton );
        buttons.add( cancelButton );
        
        Container content = getContentPane();
        content.setLayout( new GridBagLayout() );
        
        Insets insets = new Insets( 0, 0, 0, 0 );
        
        JLabel info = new JLabel( "Select or define new id for Dockable" );
        content.add( info, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.FIRST_LINE_START, GridBagConstraints.NONE, 
                insets, 0, 0 ));
        
        content.add( name, new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.LINE_START, GridBagConstraints.HORIZONTAL,
                insets, 0, 0 ));
        
        content.add( new JScrollPane( knownNames ), new GridBagConstraints( 0, 2, 1, 1, 1.0, 100.0,
                GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                insets, 0, 0 ));
        
        content.add( buttons, new GridBagConstraints( 0, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.LAST_LINE_END, GridBagConstraints.NONE,
                insets, 0, 0 ));
        
        pack();
        setLocationRelativeTo( frame );
    }
    
    public void openDialog(){
        DefaultListModel model = new DefaultListModel();
        String[] names = frame.listEmptyIds();
        Arrays.sort( names );
        for( String name : names ){
            model.addElement( name );
        }
        knownNames.setModel( model );
        if( names.length > 0 ){
            knownNames.removeSelectionInterval( 0, names.length-1 );
        }
        name.setText( "" );
        
        updateEnabledState();
        setVisible( true );
    }
    
    public void updateEnabledState(){
        String[] list = frame.listUsedIds();
        String name = this.name.getText();
        
        boolean state = name.length() > 0;
        
        if( state ){
            for( String check : list ){
                if( name.equals( check )){
                    state = false;
                    break;
                }
            }
        }
        
        okButton.setEnabled( state );
    }
    
    public void okPressed(){
        setVisible( false );
        String id = name.getText();
        TestDockable dockable = new TestDockable( id );
        
        frame.add( id, dockable );
    }
    
    public void cancelPressed(){
        setVisible( false );
    }
}
